#include "disk.hpp"

using namespace std;

disk::disk(DDSDomainParticipant *participant) : participant_ (participant)
{
    //Getting QoS files
    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/disk_publisher_qos.xml";


    //TODO set QoS configuration to Publisher, Topic and DataWriter
    
    instance_handle_ = DDS_HANDLE_NIL;
    publisher_ = participant_->create_publisher(
        DDS_PUBLISHER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);    	

    if (publisher_ == NULL) {
        syslog(LOG_ERR, "Disk plugin: create_publisher error");
    }
	
	/* Register type before creating topic */
	type_name_ = filesystemTypeSupport::get_type_name();
	retcode_ = filesystemTypeSupport::register_type(
		participant_, type_name_);
		
	if (retcode_ != DDS_RETCODE_OK) {
		syslog(LOG_ERR, "Disk plugin: register_type error %d", retcode_);
	}  
	
	topic_ = participant_->create_topic(
		"FileSystem",
        	type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
	        DDS_STATUS_MASK_NONE);
	if (topic_ == NULL) {
        syslog(LOG_ERR, "Disk plugin: create_topic error");
	}
	
	/* To customize data writer QoS, use 
       	the configuration file USER_QOS_PROFILES.xml */
	writer_ = publisher_->create_datawriter(
        	topic_, DDS_DATAWRITER_QOS_DEFAULT, NULL /* listener */,
        	DDS_STATUS_MASK_NONE);
	
	if (writer_ == NULL) {
		syslog(LOG_ERR, "Disk plugin: create_datawriter error");
	}
	
	filesystem_writer_ = filesystemDataWriter::narrow(writer_);
    if (filesystem_writer_ == NULL) {
        syslog(LOG_ERR, "Disk plugin: DataWriter narrow error");
	}

    /* Create data sample for writing */
    instance_ = filesystemTypeSupport::create_data();
    if (instance_ == NULL) {
        syslog(LOG_ERR, "Disk plugin: filesystemTypeSupport::create_data error");
    }
    

    glibtop_init();
    static struct utsname name; 
	uname(&name);            

	hi_.hostname = name.nodename;
	hi_.sysname = name.sysname;
	hi_.release_name =  name.release;
	hi_.machine = name.machine; 

}


void disk::publish_information(void)
{

	glibtop_mountentry *entries;
	glibtop_mountlist mountlist;


	entries = glibtop_get_mountlist(&mountlist, 0);

	for (int i = 0; i != mountlist.number; ++i) {
		if (string(entries[i].devname).find("/dev/") == 0 || string(entries[i].mountdir).find("/media/") == 0) {
			glibtop_fsusage usage;
			glibtop_get_fsusage(&usage, entries[i].mountdir);
			
			strcpy(instance_->name, entries[i].devname);
			strcpy(instance_->mountdir, entries[i].mountdir);
			instance_->total = ((usage.blocks * usage.block_size) / 1024);
			instance_->used = ((usage.blocks - usage.bfree) * usage.block_size) / 1024;
			instance_->free = ((usage.bavail * usage.block_size) / 1024);
			instance_->used_per = (usage.blocks - usage.bavail) * 100.0/ (double)usage.blocks;
			instance_->free_per = instance_->free* 100.0 / (double)instance_->total;
			instance_->read = usage.read;
			instance_->write = usage.write;
            instance_->host = hi_;
			
            retcode_ = filesystem_writer_->write(*instance_, instance_handle_);
            if (retcode_ != DDS_RETCODE_OK) {
                syslog(LOG_ERR, "Disk plugin: write error %d", retcode_);
            }
		}

	}

	g_free(entries);
}



