#include "net_load.hpp"

using namespace std;

net_load::net_load(DDSDomainParticipant *participant) : participant_(participant)
{
    //Getting QoS files
    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/net_load_publisher_qos.xml";

    //TODO set QoS configuration to Publisher, Topic and DataWriter
    
    instance_handle_ = DDS_HANDLE_NIL;
    publisher_ = participant_->create_publisher(
        DDS_PUBLISHER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);    	

    if (publisher_ == NULL) {
        syslog(LOG_ERR,"NetLoad plugin: create_publisher error");
    }
	
	/* Register type before creating topic */
	type_name_ = netloadTypeSupport::get_type_name();
	retcode_ = netloadTypeSupport::register_type(
		participant_, type_name_);
		
	if (retcode_ != DDS_RETCODE_OK) {
		syslog(LOG_ERR, "NetLoad plugin: register_type error %d", retcode_);
	}  
	
	topic_ = participant_->create_topic(
		"net_load",
        	type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
	        DDS_STATUS_MASK_NONE);
	if (topic_ == NULL) {
        syslog(LOG_ERR,"NetLoad plugin: create_topic error");
	}
	
	/* To customize data writer QoS, use 
       	the configuration file USER_QOS_PROFILES.xml */
	writer_ = publisher_->create_datawriter(
        	topic_, DDS_DATAWRITER_QOS_DEFAULT, NULL /* listener */,
        	DDS_STATUS_MASK_NONE);
	if (writer_ == NULL) {
		syslog(LOG_ERR,"NetLoad plugin: create_datawriter error");
	}
	
	netload_writer_ = netloadDataWriter::narrow(writer_);
    if (netload_writer_ == NULL) {
        syslog(LOG_ERR,"NetLoad plugin: DataWriter narrow error");
	}

    /* Create data sample for writing */
    instance_ = netloadTypeSupport::create_data();
    if (instance_ == NULL) {
        syslog(LOG_ERR,"NetLoad plugin: netloadTypeSupport::create_data error");
    }
    
    glibtop_init();
    static struct utsname name; 
	uname(&name);            

	hi_.hostname = name.nodename;
	hi_.sysname = name.sysname;
	hi_.release_name =  name.release;
	hi_.machine = name.machine;     
    
}    


void net_load::publish_information()
{
    devices_ = glibtop_get_netlist(&buf_);
    
    for(int i = 0; i < buf_.number; ++i) {
        glibtop_get_netload (&netload_, devices_[i]);

        strcpy(instance_->device, devices_[i]);
        instance_->flags = netload_.flags;
        instance_->if_flags = netload_.if_flags;

        //Dealing with network addresses
        addr_.s_addr = netload_.address;
        subnet_.s_addr = netload_.subnet;
        strcpy(instance_->address_str, inet_ntoa(addr_));
        strcpy(instance_->subnet_str, inet_ntoa(subnet_));
        
        instance_->mtu = netload_.mtu;
        instance_->collisions = netload_.collisions;
        instance_->packets_in = netload_.packets_in;
        instance_->packets_out = netload_.packets_out;
        instance_->packets_total = netload_.packets_total;
        instance_->bytes_in = netload_.bytes_in;
        instance_->bytes_out = netload_.bytes_out;
        instance_->bytes_total = netload_.bytes_total;  
        instance_->errors_in = netload_.errors_in;
        instance_->errors_out = netload_.errors_out;
        instance_->errors_total = netload_.errors_total;

        char hwaddres_char[32];
        sprintf(hwaddres_char,"%02x:%02x:%02x:%02x:%02x:%02x",
            (netload_.hwaddress)[0]  & 0xff,
            (netload_.hwaddress)[1]  & 0xff,
            (netload_.hwaddress)[2]  & 0xff,
            (netload_.hwaddress)[3]  & 0xff,
            (netload_.hwaddress)[4]  & 0xff,
            (netload_.hwaddress)[5]  & 0xff);
        
        strcpy(instance_->hwaddress_str, hwaddres_char);
        instance_->host = hi_;

        retcode_ = netload_writer_->write(*instance_, instance_handle_);
        if (retcode_ != DDS_RETCODE_OK) {
            syslog(LOG_ERR,"NetLoad plugin: write error %d", retcode_);
        }

    }
	
}

